package com.link.ext.basic.service;

import com.link.core.basic.service.BasicServiceException;
import com.link.core.basic.service.BasicServiceImpl;
import com.link.core.util.StringUtils;
import com.link.ext.basic.dao.mapper.BasicExtMapper;
import com.link.ext.basic.model.BasicBaseModel;
import com.link.ext.basic.model.BasicExtModel;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.springframework.transaction.annotation.Transactional;

public class BasicExtServiceImpl<T extends BasicBaseModel> extends BasicServiceImpl<T> implements BasicExtService<T> {
   public BasicExtMapper<T> getBasicMapper() throws Exception {
      throw new BasicServiceException("获取Basic Mapper 失败，请确认程序是否正确！");
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void insert(T entity) throws Exception {
      super.insert(entity);
      BasicBaseModel model = entity;
      if (model.getExtFlag() && model.getExtList() != null) {
         List<BasicExtModel> extList = model.getExtList();

         for(int i = 0; i < extList.size(); ++i) {
            BasicExtModel ext = (BasicExtModel)extList.get(i);
            if (!StringUtils.isBlank(ext.getValue())) {
               ext.setParentId(entity.getId());
               ext.setId(this.keyGenerateService.keyGenerate());
               this.getBasicMapper().extInsert(ext);
            }
         }
      }

   }

   public void update(T entity) throws Exception {
      super.update(entity);
      BasicBaseModel model = entity;
      if (model.getExtFlag() && model.getExtList() != null) {
         List<BasicExtModel> extList = model.getExtList();

         for(int i = 0; i < extList.size(); ++i) {
            BasicExtModel ext = (BasicExtModel)extList.get(i);
            ext.setParentId(entity.getId());
            ext.setId(this.keyGenerateService.keyGenerate());
            int count = this.getBasicMapper().extUpdate(ext);
            if (count < 1 && !StringUtils.isBlank(ext.getValue())) {
               ext.setId(this.keyGenerateService.keyGenerate());
               this.getBasicMapper().extInsert(ext);
            }
         }
      }

   }

   public T queryById(T entity) throws Exception {
      T record = (BasicBaseModel)super.queryById(entity);
      if (entity.getExtFlag() && record != null) {
         String[] parentIds = new String[]{record.getId()};
         String[] names = entity.getExtFields() != null ? entity.getExtFields().split(",") : null;
         BasicExtModel extExample = new BasicExtModel();
         extExample.setParentIds(parentIds);
         extExample.setPageFlag(false);
         extExample.setNames(names);
         List<BasicExtModel> extList = this.getBasicMapper().queryExtByExamplePage(extExample);
         record.setExtList(extList);
      }

      return record;
   }

   @Transactional(
      rollbackFor = {BasicServiceException.class}
   )
   public void deleteById(T entity) throws Exception {
      super.deleteById(entity);
      if (entity.getExtFlag()) {
         BasicExtModel ext = new BasicExtModel();
         ext.setParentId(entity.getId());
         int rows = this.getBasicMapper().deleteExtByParentId(ext);
         System.out.println("delete ext rows:" + rows);
      }

   }

   public List<T> queryAll(T entity) throws Exception {
      List<T> list = super.queryAll(entity);
      return this.setExtResultToBaseResult(entity, list);
   }

   private List<T> setExtResultToBaseResult(T entity, List<T> list) throws Exception {
      if (list != null && list.size() > 0 && entity.getExtFlag()) {
         int startIndex = false;
         int pageRows = 500;
         String[] extFields = null;
         if (entity.getExtFields() != null) {
            extFields = entity.getExtFields().split(",");
         }

         Map<String, Integer> map = new HashMap(list.size());

         int end;
         for(int startIndex = 0; startIndex < list.size(); startIndex = end) {
            int begin = startIndex;
            end = begin + pageRows;
            end = Math.min(end, list.size());
            int size = end - begin;
            System.err.println("begin: " + begin + ", end: " + end);
            String[] parentIds = new String[size];

            for(int i = begin; i < end; ++i) {
               map.put(((BasicBaseModel)list.get(i)).getId(), i);
               parentIds[i - begin] = ((BasicBaseModel)list.get(i)).getId();
            }

            BasicExtModel extExample = new BasicExtModel();
            extExample.setParentIds(parentIds);
            extExample.setPageFlag(false);
            extExample.setNames(extFields);
            List<BasicExtModel> extList = this.getBasicMapper().queryExtByExamplePage(extExample);

            for(int i = 0; extList != null && i < extList.size(); ++i) {
               String parentId = ((BasicExtModel)extList.get(i)).getParentId();
               int index = (Integer)map.get(parentId);
               BasicBaseModel base = (BasicBaseModel)list.get(index);
               BasicExtModel ext = (BasicExtModel)extList.get(i);
               List<BasicExtModel> recordExtList = base.getExtList();
               if (recordExtList == null) {
                  List<BasicExtModel> recordExtList = new ArrayList();
                  recordExtList.add(ext);
                  base.setExtList(recordExtList);
               } else {
                  recordExtList.add(ext);
               }
            }
         }
      }

      return list;
   }

   public List<T> queryByExamplePage(T entity) throws Exception {
      List<T> list = super.queryByExamplePage(entity);
      return this.setExtResultToBaseResult(entity, list);
   }
}
